<?php

namespace EventPortal\Domain\Agenda;

use Carbon\Carbon;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;

class ActiveTabTracker
{
    private bool $hasActiveTab = false;

    private Collection $tabs;

    /**
     * Takes a Collection of Carbon dates.
     */
    public function __construct(Collection $dates)
    {
        $this->tabs = $dates
            ->sort()
            ->mapWithKeys(function (Carbon $date) {
                return [
                    $date->format('Y-m-d') => [
                        'isActive' => $this->setIsActive($date),
                        'date' => $date,
                    ],
                ];
            });

        if ($this->hasActiveTab) {
            return;
        }

        $this->makeLastTabActive();
    }

    public function isActive(Carbon $date): bool
    {
        $key = $date->format('Y-m-d');

        if (Arr::has($this->tabs, $key) === false) {
            return false;
        }

        $tab = Arr::get($this->tabs, $key);

        return Arr::get($tab, 'isActive');
    }

    /**
     * Determine if date is active.
     */
    private function setIsActive(Carbon $date): bool
    {
        if ($this->hasActiveTab) {
            return false;
        }

        if ($date->isToday() || $date->isFuture()) {
            $this->hasActiveTab = true;

            return true;
        }

        return false;
    }

    private function makeLastTabActive(): void
    {
        $lastDate = $this->tabs->last()['date'];

        $this->tabs = $this->tabs->replace([
            $lastDate->format('Y-m-d') => [
                'isActive' => true,
                'date' => $lastDate,
            ],
        ]);
    }
}
