<?php

namespace EventPortal\Domain\Util\Modal;

use Illuminate\Support\Arr;

abstract class Modal
{
    /** @var string */
    public $type;

    /** @var \Illuminate\Database\Eloquent\Model */
    public $context;

    /** @var null|string */
    public $content;

    /** @var array */
    public $actions = [];

    public function __construct(string $type)
    {
        $this->type = $type;
    }

    /**
     * @param null|mixed $content
     */
    public function content($content = null): self
    {
        $this->content = $content;

        return $this;
    }

    /**
     * @param null|mixed $context
     */
    public function context($context = null): self
    {
        $this->context = $context;

        return $this;
    }

    /**
     * @param null|array|callable $actions
     * @return $this
     */
    public function actions($actions = null): self
    {
        $this->actions[] = $actions ?? [];

        return $this;
    }

    protected function resolve(): void
    {
        if ($this->actions === []) {
            return;
        }

        $buttons = array_map(function ($button) {
            return app()->call($button, Arr::wrap($this->context));
        }, $this->actions);

        $this->actions = array_merge(...$buttons);
    }
}
